( cd usr/lib ; rm -rf libdc1394_control.so.12 )
( cd usr/lib ; ln -sf libdc1394_control.so.12.1.2 libdc1394_control.so.12 )
( cd usr/lib ; rm -rf libdc1394_control.so )
( cd usr/lib ; ln -sf libdc1394_control.so.12.1.2 libdc1394_control.so )
